@@ -1,5 +1,6 @@
 "egscore" <-
-function(formula,data,snpsubset,idsubset,kinship.matrix,naxes=3,strata,times=1,quiet=FALSE,bcast=10,clambda=TRUE,propPs=1.0) {
+function(formula,data,snpsubset,idsubset,kinship.matrix,naxes=3,strata,
+		times=1,quiet=FALSE,bcast=10,clambda=TRUE,propPs=1.0) {
   	if (!is(data,"gwaa.data")) {
 		stop("wrong data class: should be gwaa.data")
   	}
@@ -53,6 +54,7 @@ function(formula,data,snpsubset,idsubset,kinship.matrix,naxes=3,strata,times=1,q
 	tmp <- dim(kinship.matrix)
 	if (tmp[1] == tmp[2]) {
 		tmp <- t(kinship.matrix)
+		diag(kinship.matrix) <- attr(kinship.matrix,"Var")
 		kinship.matrix[upper.tri(kinship.matrix)] <- tmp[upper.tri(tmp)]
 		ev <- eigen(kinship.matrix,symmetric=TRUE)$vectors
 		rownames(ev) <- rownames(kinship.matrix)
@@ -68,7 +70,7 @@ function(formula,data,snpsubset,idsubset,kinship.matrix,naxes=3,strata,times=1,q
 	resid <- lm(resid~ev)$resid
 
 	lenn <- data@gtdata@nsnps;
-	out <- list()
+	#out <- list()
 	for (j in c(1:(times+1*(times>1)))) {
 		if (j>1) resid <- sample(resid,replace=FALSE)
 		chi2 <- .C("egscore",as.raw(data@gtdata@gtps),as.double(resid),as.integer(naxes),as.double(ev),as.integer(data@gtdata@nids),as.integer(data@gtdata@nsnps), as.integer(nstra), as.integer(strata), chi2 = double(7*data@gtdata@nsnps), PACKAGE="GenABEL")$chi2
@@ -83,9 +85,9 @@ function(formula,data,snpsubset,idsubset,kinship.matrix,naxes=3,strata,times=1,q
 #		}
 		if (j == 1) {
 			chi2.1df <- chi2[1:lenn];
-			out$chi2.1df <- chi2.1df
+			###out$chi2.1df <- chi2.1df
 			chi2.2df <- rep(NA,lenn) #chi2[(lenn+1):(2*lenn)];
-			out$chi2.2df <- chi2.2df
+			###out$chi2.2df <- chi2.2df
 			actdf <- chi2[(2*lenn+1):(3*lenn)];
 			lambda <- list()
 			if (is.logical(clambda)) {
@@ -138,28 +140,50 @@ function(formula,data,snpsubset,idsubset,kinship.matrix,naxes=3,strata,times=1,q
 	if (times > bcast) cat("\n")
 
 	if (times>1) {
-		out$P1df <- pr.1df/times
-		out$P1df <- replace(out$P1df,(out$P1df==0),1/(1+times))
-		out$P2df <- pr.2df/times
-		out$P2df <- replace(out$P2df,(out$P2df==0),1/(1+times))
-		out$Pc1df <- pr.c1df/times
-		out$Pc1df <- replace(out$Pc1df,(out$Pc1df==0),1/(1+times))
+		P1df <- pr.1df/times
+		P1df <- replace(P1df,(P1df==0),1/(1+times))
+		P2df <- pr.2df/times
+		P2df <- replace(P2df,(P2df==0),1/(1+times))
+		Pc1df <- pr.c1df/times
+		Pc1df <- replace(Pc1df,(Pc1df==0),1/(1+times))
 	} else {
-		out$P1df <- pchisq(chi2.1df,1,lower=F)
-		out$P2df <- pchisq(chi2.2df,actdf,lower=F)
-		out$Pc1df <- pchisq(chi2.c1df,1,lower=F)
+		P1df <- pchisq(chi2.1df,1,lower=F)
+		P2df <- pchisq(chi2.2df,actdf,lower=F)
+		Pc1df <- pchisq(chi2.c1df,1,lower=F)
 	}
-	out$lambda <- lambda
-	out$effB <- effB
-	out$effAB <- effAB
-	out$effBB <- effBB
-	out$snpnames <- data@gtdata@snpnames
-	out$map <- data@gtdata@map
-	out$chromosome <- data@gtdata@chromosome
-	out$idnames <- data@gtdata@idnames
-	out$formula <- match.call()
-	out$family <- paste("score test for association, eigen-adjustment")
-	out$N <- chi2[(6*lenn+1):(lenn*7)]
-	class(out) <- "scan.gwaa"
+	#out$lambda <- lambda
+	#out$effB <- effB
+	#out$effAB <- effAB
+	#out$effBB <- effBB
+	#out$snpnames <- data@gtdata@snpnames
+	#out$map <- data@gtdata@map
+	#out$chromosome <- data@gtdata@chromosome
+	#out$idnames <- data@gtdata@idnames
+	#out$formula <- match.call()
+	#out$family <- paste("score test for association, eigen-adjustment")
+	#out$N <- chi2[(6*lenn+1):(lenn*7)]
+	#class(out) <- "scan.gwaa"
+	#out
+	if (is.null(Pc1df)) {
+		results <- data.frame(N=chi2[(6*lenn+1):(lenn*7)],
+				effB = effB, se_effB = effB/sqrt(chi2.1df), chi2.1df = chi2.1df, P1df = P1df, 
+				effAB=effAB, effBB=effBB, chi2.2df = chi2.2df, P2df = P2df,
+				stringsAsFactors = FALSE)
+	} else {
+		results <- data.frame(N=chi2[(6*lenn+1):(lenn*7)],
+				effB = effB, se_effB = effB/sqrt(chi2.1df), chi2.1df = chi2.1df, P1df = P1df, 
+				Pc1df = Pc1df, 
+				effAB=effAB, effBB=effBB, chi2.2df = chi2.2df, P2df = P2df,
+				stringsAsFactors = FALSE)
+	}
+	rownames(results) <- snpnames(data)
+	out <- new("scan.gwaa",
+			results=results,
+			annotation = annotation(data), 
+			lambda = lambda,
+			idnames = idnames(data), 
+			call = match.call(), 
+			family = "EIGENSCORETEST"
+	) 
 	out
 }
