@@ -1,5 +1,6 @@
 "mmscore" <-
-function(h2object,data,snpsubset,idsubset,strata,times=1,quiet=FALSE,bcast=10,clambda=TRUE,propPs=1.0) {
+function(h2object,data,snpsubset,idsubset,strata,times=1,quiet=FALSE,
+		bcast=10,clambda=TRUE,propPs=1.0) {
   	if (is(data,"gwaa.data")) 
 	{
 		checkphengen(data)
@@ -52,8 +53,8 @@ function(h2object,data,snpsubset,idsubset,strata,times=1,quiet=FALSE,bcast=10,cl
 		if (j == 1) {
 			chi2.1df <- chi2[1:lenn];
 			chi2.2df <- chi2[(lenn+1):(2*lenn)];
-			out$chi2.1df <- chi2.1df
-			out$chi2.2df <- chi2.2df
+			#out$chi2.1df <- chi2.1df
+			#out$chi2.2df <- chi2.2df
 			actdf <- chi2[(2*lenn+1):(3*lenn)];
 			lambda <- list()
 			if (is.logical(clambda)) {
@@ -106,31 +107,53 @@ function(h2object,data,snpsubset,idsubset,strata,times=1,quiet=FALSE,bcast=10,cl
 	if (times > bcast) cat("\n")
 
 	if (times>1) {
-		out$P1df <- pr.1df/times
-		out$P1df <- replace(out$P1df,(out$P1df==0),1/(1+times))
+		P1df <- pr.1df/times
+		P1df <- replace(P1df,(P1df==0),1/(1+times))
 #		out$P2df <- pr.2df/times
 #		out$P2df <- replace(out$P2df,(out$P2df==0),1/(1+times))
-		out$Pc1df <- pr.c1df/times
+		Pc1df <- pr.c1df/times
 #		out$Pc1df <- replace(out$Pc1df,(out$Pc1df==0),1/(1+times))
 	} else {
-		out$P1df <- pchisq(chi2.1df,1,lower=F)
+		P1df <- pchisq(chi2.1df,1,lower=F)
 #		out$P2df <- pchisq(chi2.2df,actdf,lower=F)
-		out$Pc1df <- pchisq(chi2.c1df,1,lower=F)
+		Pc1df <- pchisq(chi2.c1df,1,lower=F)
 	}
-	out$lambda <- lambda
-	out$effB <- effB #*var(h2object$residualY,na.rm=T)
+	#out$lambda <- lambda
+	#out$effB <- effB #*var(h2object$residualY,na.rm=T)
 #	out$effAB <- effAB
 #	out$effBB <- effBB
-	out$snpnames <- data@snpnames
-	out$map <- data@map
-	out$chromosome <- data@chromosome
-	out$idnames <- data@idnames
-	out$formula <- match.call()
-	out$family <- paste("score test for association with trait type") #,trait.type)
-	out$effAB <- rep(NA,length(out$P1df))
-	out$effBB <- rep(NA,length(out$P1df))
-	out$P2df <- rep(NA,length(out$P1df))
-	out$N <- chi2[(6*lenn+1):(lenn*7)]
-	class(out) <- "scan.gwaa"
+	#out$snpnames <- data@snpnames
+	#out$map <- data@map
+	#out$chromosome <- data@chromosome
+	#out$idnames <- data@idnames
+	#out$formula <- match.call()
+	#out$family <- paste("score test for association with trait type") #,trait.type)
+	effAB <- rep(NA,length(P1df))
+	effBB <- rep(NA,length(P1df))
+	P2df <- rep(NA,length(P1df))
+	#out$N <- chi2[(6*lenn+1):(lenn*7)]
+	#class(out) <- "scan.gwaa"
+	#out
+	if (is.null(Pc1df)) {
+		results <- data.frame(N=chi2[(6*lenn+1):(lenn*7)],
+				effB = effB, se_effB = effB/sqrt(chi2.1df), chi2.1df = chi2.1df, P1df = P1df, 
+				effAB=effAB, effBB=effBB, chi2.2df = chi2.2df, P2df = P2df,
+				stringsAsFactors = FALSE)
+	} else {
+		results <- data.frame(N=chi2[(6*lenn+1):(lenn*7)],
+				effB = effB, se_effB = effB/sqrt(chi2.1df), chi2.1df = chi2.1df, P1df = P1df, 
+				Pc1df = Pc1df, 
+				effAB=effAB, effBB=effBB, chi2.2df = chi2.2df, P2df = P2df,
+				stringsAsFactors = FALSE)
+	}
+	rownames(results) <- snpnames(data)
+	out <- new("scan.gwaa",
+			results=results,
+			annotation = annotation(data), 
+			lambda = lambda,
+			idnames = idnames(data), 
+			call = match.call(), 
+			family = "mmscore"
+	) 
 	out
 }
